import { Config, ExtractActionsType, ExtractSelectorsType, ExtractStateType, EntityFeatureFactory, FeatureSelectors, KeyedConfig, TraitActions, TraitFactory, TraitSelectors, TraitStateMutators, UnionToIntersection, TraitActionsFactory, TraitSelectorsFactory, TraitInitialStateFactory, TraitStateMutatorsFactory, TraitReducerFactory, TraitEffectsFactory } from './model';
import { ActionType } from '@ngrx/store';
import { Type } from './local-store';
export declare function createTraitFactory<State = {}, A extends TraitActions = {}, S extends TraitSelectors<State> = {}, M extends TraitStateMutators<State> = {}, KEY extends string = string, C = unknown, KC = KeyedConfig<KEY, C>>(f: {
    key: KEY;
    config?: C;
    depends?: string[];
    actions?: TraitActionsFactory<A, KC>;
    selectors?: TraitSelectorsFactory<State, S, KC>;
    initialState?: TraitInitialStateFactory<State, KC>;
    mutators?: TraitStateMutatorsFactory<State, M, KC>;
    reducer?: TraitReducerFactory<State, A, S, M, KC>;
    effects?: TraitEffectsFactory<State, A, S, KC>;
}): TraitFactory<State, A, S, M, KEY, C, KC>;
export declare function createEntityFeatureFactory<F extends readonly TraitFactory[], EntityName extends string, EntitiesName extends string = `${EntityName}s`>({ entityName, entitiesName, }: {
    entityName: EntityName;
    entitiesName?: EntitiesName;
}, ...traits: F): EntityFeatureFactory<EntityName, EntitiesName, ExtractStateType<F>, ExtractActionsType<F>, ExtractSelectorsType<F>>;
export declare function setPropertyReducer<State, P extends keyof State>(sourceReducer: (state: State, action: ActionType<any>) => State, property: P, propertyReducer: (state: State[P], action: ActionType<any>) => State[P]): (state: State, action: ActionType<any>) => State;
export declare function setPropertiesReducer<State, P extends keyof State>(sourceReducer: (state: State, action: ActionType<any>) => State, propertiesReducers: {
    [key in P]: (state: State[P], action: ActionType<any>) => State[P];
}): (state: State, action: ActionType<any>) => State;
export declare function joinReducers<State>(firstReducer: (state: State, action: ActionType<any>) => State, secondReducer: (state: any, action: ActionType<any>) => any): (state: State, action: ActionType<any>) => State;
export declare function combineEntityFeatures<T extends {
    [key: string]: EntityFeatureFactory<any, any>;
}, K extends keyof T, State extends {
    [P in K]: ExtractStateType<ReturnType<T[P]>>;
}, A extends {
    [P in K]: ExtractActionsType<ReturnType<T[P]>>;
}, S extends {
    [P in K]: FeatureSelectors<State, ExtractSelectorsType<ReturnType<T[P]>>>;
}, R extends (config: Config<State>) => {
    actions: A;
    selectors: S;
    reducer: (state: State, action: ActionType<any>) => State;
    effects: Type<any>[];
    initialState: State;
}>(traitFactoriesMap: T): R;
export declare function mixEntityFeatures<T extends {
    [key: string]: EntityFeatureFactory<any, any>;
}, K extends keyof T, State extends {
    [P in K]: ExtractStateType<ReturnType<T[P]>>;
}, A extends TraitActions & UnionToIntersection<ExtractActionsType<ReturnType<T[K]>>>, S extends TraitSelectors<any> & UnionToIntersection<FeatureSelectors<State, ExtractSelectorsType<ReturnType<T[K]>>>>, R extends EntityFeatureFactory<any, any, State, A, S>>(traitFactoriesMap: T): R;
export declare function addEntityFeaturesProperties<F extends EntityFeatureFactory<any, any>, T extends {
    [key: string]: EntityFeatureFactory<any, any, any, any, any>;
}, K extends keyof T, State extends ExtractStateType<ReturnType<F>> & {
    [P in K]: ExtractStateType<ReturnType<T[P]>>;
}, A extends ExtractActionsType<ReturnType<F>> & {
    [P in K]: ExtractActionsType<ReturnType<T[P]>>;
}, S extends FeatureSelectors<State, ExtractSelectorsType<ReturnType<F>>> & {
    [P in K]: FeatureSelectors<State, ExtractSelectorsType<ReturnType<T[P]>>>;
}, R extends (config: Config<State>) => {
    actions: A;
    selectors: S;
    reducer: (state: State, action: ActionType<any>) => State;
    effects: Type<any>[];
    initialState: State;
}>(targetTraitFactory: F, traitFactoriesMap: T): R;
