import { combineReducers, createFeatureSelector, createReducer, createSelector, } from '@ngrx/store';
import { capitalize } from './util';
export function createTraitFactory(f) {
    return f;
}
export function createEntityFeatureFactory({ entityName, entitiesName, }, ...traits) {
    return ((config) => {
        const singular = capitalize(entityName);
        const plural = entitiesName
            ? capitalize(entitiesName)
            : capitalize(entityName + 's');
        const sortedTraits = sortTraits([...traits]);
        const allConfigs = buildAllConfigs(sortedTraits);
        const allActions = buildAllActions(sortedTraits, config.actionsGroupKey, singular, plural, allConfigs);
        const allSelectors = buildAllSelectors(sortedTraits, allConfigs);
        const allMutators = buildAllMutators(sortedTraits, allSelectors, allConfigs);
        const initialState = buildInitialState(sortedTraits, allConfigs);
        const reducer = buildReducer(sortedTraits, initialState, allActions, allSelectors, allMutators, allConfigs);
        const featureSelector = typeof config.featureSelector === 'string'
            ? createFeatureSelector(config.featureSelector)
            : config.featureSelector;
        const allFeatureSelectors = allSelectors && getSelectorsForFeature(featureSelector, allSelectors);
        const allEffects = buildAllEffects(sortedTraits, allActions, allFeatureSelectors, allConfigs);
        return {
            actions: entityName
                ? renameProps(allActions, singular, plural)
                : allActions,
            selectors: entityName
                ? renameProps(allFeatureSelectors, singular, plural)
                : allSelectors,
            initialState,
            reducer: reducer !== null && reducer !== void 0 ? reducer : createReducer(initialState),
            effects: allEffects,
        };
    });
}
function renameProps(target, entityName, entitiesName) {
    const result = {};
    for (const [key, value] of Object.entries(target)) {
        const newKey = key
            .replace('Entities', entitiesName)
            .replace('Entity', entityName);
        result[newKey] = value;
    }
    return result;
}
function sortTraits(traits) {
    var _a;
    const sortedTraits = [];
    for (let i = 0; i < traits.length; i++) {
        const trait = traits[i];
        if (!((_a = trait.depends) === null || _a === void 0 ? void 0 : _a.length)) {
            sortedTraits.push(trait);
            continue;
        }
        if (trait.depends.length > 1)
            for (const d of trait.depends) {
                const isTraitPresent = traits.some((tr) => tr.key === d);
                if (isTraitPresent) {
                    trait.depends = [d];
                    break;
                }
            }
        if (trait.depends.length > 1)
            throw Error('could not find dependencies ' + trait.depends.join(' '));
        const isDependencyAlreadyAdded = sortedTraits.some((tr) => { var _a; return tr.key === ((_a = trait === null || trait === void 0 ? void 0 : trait.depends) === null || _a === void 0 ? void 0 : _a[0]); });
        if (isDependencyAlreadyAdded)
            sortedTraits.push(trait);
        else {
            // move trait to the end
            delete traits[i];
            traits.push(trait);
        }
    }
    return sortedTraits;
}
function buildAllConfigs(sortedTraits) {
    return sortedTraits.reduce((acc, factory) => {
        acc[factory.key] = factory.config;
        return acc;
    }, {});
}
function buildAllActions(sortedTraits, actionsGroupKey, entityName, entitiesName, allConfigs) {
    return sortedTraits.reduce((previousResult, factory) => {
        var _a, _b;
        let result = (_b = (_a = factory === null || factory === void 0 ? void 0 : factory.actions) === null || _a === void 0 ? void 0 : _a.call(factory, {
            actionsGroupKey: actionsGroupKey,
            entityName,
            entitiesName,
            allConfigs,
        })) !== null && _b !== void 0 ? _b : {};
        result = previousResult ? Object.assign(Object.assign({}, previousResult), result) : result;
        return result;
    }, {});
}
function buildAllSelectors(sortedTraits, allConfigs) {
    return sortedTraits.reduce((previousResult, factory) => {
        var _a, _b;
        let result = (_b = (_a = factory === null || factory === void 0 ? void 0 : factory.selectors) === null || _a === void 0 ? void 0 : _a.call(factory, {
            previousSelectors: previousResult,
            allConfigs,
        })) !== null && _b !== void 0 ? _b : {};
        result = previousResult ? Object.assign(Object.assign({}, previousResult), result) : result;
        return result;
    }, {});
}
function buildAllMutators(sortedTraits, allSelectors, allConfigs) {
    return (sortedTraits.reduce((previousResult, factory) => {
        var _a, _b;
        let result = (_b = (_a = factory === null || factory === void 0 ? void 0 : factory.mutators) === null || _a === void 0 ? void 0 : _a.call(factory, {
            allSelectors: allSelectors,
            previousMutators: previousResult,
            allConfigs,
        })) !== null && _b !== void 0 ? _b : {};
        result = previousResult ? Object.assign(Object.assign({}, previousResult), result) : result;
        return result;
    }, {}) || {});
}
function buildInitialState(sortedTraits, allConfigs) {
    return sortedTraits.reduce((previousResult, factory) => {
        var _a, _b, _c;
        const result = (_c = (_b = (_a = factory === null || factory === void 0 ? void 0 : factory.initialState) === null || _a === void 0 ? void 0 : _a.call(factory, {
            previousInitialState: previousResult,
            allConfigs,
        })) !== null && _b !== void 0 ? _b : previousResult) !== null && _c !== void 0 ? _c : {};
        return result;
    }, {});
}
function buildReducer(sortedTraits, initialState, allActions, allSelectors, allMutators, allConfigs) {
    return sortedTraits.reduce((previousResult, factory) => {
        var _a;
        const result = (_a = factory === null || factory === void 0 ? void 0 : factory.reducer) === null || _a === void 0 ? void 0 : _a.call(factory, {
            initialState,
            allActions,
            allSelectors,
            allMutators,
            allConfigs,
        });
        return result && previousResult
            ? (state = initialState, action) => {
                const aState = previousResult(state, action);
                return result(aState, action);
            }
            : result !== null && result !== void 0 ? result : previousResult;
    }, undefined);
}
function buildAllEffects(sortedTraits, allActions, allFeatureSelectors, allConfigs) {
    return sortedTraits.reduce((previousResult, factory) => {
        var _a, _b;
        let result = (_b = (_a = factory === null || factory === void 0 ? void 0 : factory.effects) === null || _a === void 0 ? void 0 : _a.call(factory, {
            allActions,
            allSelectors: allFeatureSelectors,
            allConfigs,
        })) !== null && _b !== void 0 ? _b : [];
        result = previousResult ? [...previousResult, ...result] : result;
        return result;
    }, []);
}
function getSelectorsForFeature(featureSelect, selectors) {
    const ss = {};
    for (const prop in selectors) {
        ss[prop] = createSelector(featureSelect, selectors[prop]);
    }
    return ss;
}
export function setPropertyReducer(sourceReducer, property, propertyReducer) {
    return function reducer(state, action) {
        const sourceState = sourceReducer(state, action);
        return Object.assign(Object.assign({}, sourceState), { [property]: propertyReducer(sourceState[property], action) });
    };
}
export function setPropertiesReducer(sourceReducer, propertiesReducers) {
    return function reducer(state, action) {
        const newState = Object.assign({}, sourceReducer(state, action));
        for (const property in propertiesReducers) {
            newState[property] = propertiesReducers[property](newState[property], action);
        }
        return newState;
    };
}
export function joinReducers(firstReducer, secondReducer) {
    return function reducer(state, action) {
        const sourceState = firstReducer(state, action);
        return secondReducer(sourceState, action);
    };
}
export function combineEntityFeatures(traitFactoriesMap) {
    return ((config) => {
        const featureSelector = typeof config.featureSelector === 'string'
            ? createFeatureSelector(config.featureSelector)
            : config.featureSelector;
        const actions = {};
        const selectors = {};
        const reducers = {};
        let effects = [];
        for (const [key, entityFeatureFactory] of Object.entries(traitFactoriesMap)) {
            const selector = createSelector(featureSelector, (state) => state[key]);
            const featureTraits = entityFeatureFactory({
                actionsGroupKey: config.actionsGroupKey,
                featureSelector: selector,
            });
            actions[key] = featureTraits.actions;
            selectors[key] = featureTraits.selectors;
            reducers[key] = featureTraits.reducer;
            effects = [...effects, ...featureTraits.effects];
        }
        return {
            actions,
            selectors,
            reducer: combineReducers(reducers),
            effects,
        };
    });
}
export function mixEntityFeatures(traitFactoriesMap) {
    return ((config) => {
        const featureSelector = typeof config.featureSelector === 'string'
            ? createFeatureSelector(config.featureSelector)
            : config.featureSelector;
        let actions = {};
        let selectors = {};
        const reducers = {};
        let effects = [];
        for (const [key, entityFeatureFactory] of Object.entries(traitFactoriesMap)) {
            const selector = createSelector(featureSelector, (state) => state[key]);
            const featureTraits = entityFeatureFactory({
                actionsGroupKey: config.actionsGroupKey,
                featureSelector: selector,
            });
            actions = Object.assign(Object.assign({}, actions), featureTraits.actions);
            selectors = Object.assign(Object.assign({}, selectors), featureTraits.selectors);
            reducers[key] = featureTraits.reducer;
            effects = [...effects, ...featureTraits.effects];
        }
        return {
            actions,
            selectors,
            reducer: combineReducers(reducers),
            effects,
        };
    });
}
export function addEntityFeaturesProperties(targetTraitFactory, traitFactoriesMap) {
    return ((config) => {
        const featureSelector = typeof config.featureSelector === 'string'
            ? createFeatureSelector(config.featureSelector)
            : config.featureSelector;
        const targetFeatureTraits = targetTraitFactory({
            actionsGroupKey: config.actionsGroupKey,
            featureSelector: featureSelector,
        });
        const actions = Object.assign({}, targetFeatureTraits.actions);
        const selectors = Object.assign({}, targetFeatureTraits.selectors);
        const reducers = {};
        let effects = [...targetFeatureTraits.effects];
        for (const [key, entityFeatureFactory] of Object.entries(traitFactoriesMap)) {
            const selector = createSelector(featureSelector, (state) => state[key]);
            const featureTraits = entityFeatureFactory({
                actionsGroupKey: config.actionsGroupKey,
                featureSelector: selector,
            });
            actions[key] = featureTraits.actions;
            selectors[key] = featureTraits.selectors;
            reducers[key] = featureTraits.reducer;
            effects = [...effects, ...featureTraits.effects];
        }
        return {
            actions,
            selectors,
            reducer: setPropertiesReducer(targetFeatureTraits.reducer, reducers),
            effects,
        };
    });
}
// export
/// products:{actions, selectors, }, orders: {actions,slectors}
/// { actions: {ProductActions, OrderActions}, selectors: {ProductSelectors, OrderSelectors}
// TODO finish implementing combineTrais and addPropertiesTraits
// TODO finish renaming traits
//# sourceMappingURL=create-entity-feature.js.map