import { Injectable, Injector } from '@angular/core';
import { createFeatureSelector, ReducerManager, Store } from '@ngrx/store';
import { Actions, EffectSources } from '@ngrx/effects';
import { getDestroyActionName, TraitEffect } from '../trait-effect';
import { DISABLE_LOCAL_TRAIT_EFFECTS } from './disable-local-trait-effects.token';
import * as i0 from "@angular/core";
let id = 0;
function uniqueComponentId() {
    return id++;
}
export function buildLocalTraits(injector, componentName, traitFactory, localEffect) {
    var _a;
    const reducers = injector.get(ReducerManager);
    const effects = injector.get(EffectSources);
    const store = injector.get(Store);
    const componentId = `${componentName}_${uniqueComponentId()}`;
    const traits = traitFactory({
        featureSelector: createFeatureSelector(componentId),
        actionsGroupKey: `[${componentId}]`,
    });
    traits.reducer && reducers.addReducer(componentId, traits.reducer);
    const providers = (traits.effects && [...traits.effects.map((e) => ({ provide: e }))]) || [];
    const disableLocalTraitsEffects = injector.get(DISABLE_LOCAL_TRAIT_EFFECTS, false);
    if (!disableLocalTraitsEffects) {
        const i = Injector.create({
            providers: providers,
            parent: injector,
        });
        (_a = traits.effects) === null || _a === void 0 ? void 0 : _a.forEach((e) => {
            const effect = i.get(e);
            effect.componentId = componentId;
            effects.addEffects(effect);
        });
        if (localEffect) {
            localEffect.componentId = componentId;
            queueMicrotask(() => {
                // needs to run on a microtask to give time to localEffect effects to correctly be initialized
                effects.addEffects(localEffect);
            });
        }
    }
    function destroy() {
        store.dispatch({ type: getDestroyActionName(componentId) });
        /**
         * A service that extends TraitsLocalStore and other component service are destroyed
         * before the component that depends on them, this causes that any subscriptions
         * to selectors of the TraitsLocalStore could fail because the store state is removed before
         * they are unsubscribe by the onDestroy of the component. Executing reducers.removeReducer
         * inside setTimeout ensures the state is remove after the component onDestroy was called,
         * avoiding the before mentioned possible issues.
         */
        setTimeout(() => reducers.removeReducer(componentId));
    }
    return Object.assign({ destroy }, traits);
}
export class TraitsLocalStore extends TraitEffect {
    constructor(injector) {
        super(injector.get(Actions), injector.get(Store));
        this.injector = injector;
        const config = this.setup();
        this.traits = buildLocalTraits(this.injector, config.componentName, config.traitsFactory, this);
        this.localActions = this.traits.actions;
        this.localSelectors = this.traits.selectors;
    }
    ngOnDestroy() {
        this.traits.destroy();
    }
}
TraitsLocalStore.ɵfac = i0.ɵɵngDeclareFactory({ minVersion: "12.0.0", version: "12.1.2", ngImport: i0, type: TraitsLocalStore, deps: [{ token: i0.Injector }], target: i0.ɵɵFactoryTarget.Injectable });
TraitsLocalStore.ɵprov = i0.ɵɵngDeclareInjectable({ minVersion: "12.0.0", version: "12.1.2", ngImport: i0, type: TraitsLocalStore });
i0.ɵɵngDeclareClassMetadata({ minVersion: "12.0.0", version: "12.1.2", ngImport: i0, type: TraitsLocalStore, decorators: [{
            type: Injectable
        }], ctorParameters: function () { return [{ type: i0.Injector }]; } });
//# sourceMappingURL=traits-local-store.js.map