import { createSelector } from '@ngrx/store';
import { ChangeType, } from './crud-entities.model';
export function createCrudTraitSelectors(previousSelectors) {
    function selectChanges(state) {
        return state.changes;
    }
    function selectFilteredChanges(state) {
        const cache = {};
        return state.changes.reduce((acc, value) => {
            const changes = cache[value.id];
            if (!changes) {
                cache[value.id] = [value.changeType];
                acc.push(value);
                return acc;
            }
            if (value.changeType === ChangeType.UPDATED) {
                return acc;
            }
            if (value.changeType === ChangeType.DELETED &&
                changes.includes(ChangeType.CREATED)) {
                delete cache[value.id];
                return acc.filter((v) => v.id !== value.id);
            }
            if (value.changeType === ChangeType.DELETED) {
                delete cache[value.id];
                const newAcc = acc.filter((v) => v.id !== value.id);
                newAcc.push(value);
                return newAcc;
            }
            return acc;
        }, []);
    }
    const { selectEntitiesMap } = previousSelectors;
    const selectEntitiesChangesList = createSelector((state) => selectEntitiesMap(state), selectChanges, (entities, changed, { type }) => {
        if (type)
            return changed
                .filter((c) => c.changeType === type)
                .map((change) => {
                var _a;
                return ({
                    changeType: change.changeType,
                    entity: (_a = entities[change.id]) !== null && _a !== void 0 ? _a : {
                        id: change.id,
                    },
                });
            });
        const map = changed.map((change) => {
            var _a;
            return ({
                changeType: change.changeType,
                entity: (_a = entities[change.id]) !== null && _a !== void 0 ? _a : {
                    id: change.id,
                },
            });
        });
        return map;
    });
    const selectFilteredEntitiesChangesList = createSelector(selectFilteredChanges, (state) => selectEntitiesMap(state), (changes, entities) => changes.map((c) => {
        var _a;
        return {
            entity: (_a = entities[c.id]) !== null && _a !== void 0 ? _a : { id: c.id },
            changeType: c.changeType,
        };
    }));
    return {
        selectEntitiesChangesList,
        selectFilteredEntitiesChangesList,
    };
}
//# sourceMappingURL=crud-entities.trait.selectors.js.map