import { createTraitFactory } from 'ngrx-traits';
import { createAction, createReducer, on, Store } from '@ngrx/store';
import { Injectable } from '@angular/core';
import { TraitEffect } from 'ngrx-traits';
import { Actions, createEffect, ofType } from '@ngrx/effects';
import { mapTo } from 'rxjs/operators';
import * as i0 from "@angular/core";
import * as i1 from "@ngrx/effects";
import * as i2 from "@ngrx/store";
/**
 * Generates the ngrx code needed to reset the current state to the initial state.
 * @param traitConfig - Config object fot the trait factory
 * @param traitConfig.resetOn - set an extra action which will also trigger a reset state,
 * useful if you want to create an action that reset several features states
 *
 * @example
 * // The following trait config
 *
 * export interface TestState
 * extends EntityAndStatusState<Todo>,FilterState<TodoFilter>{}
 *
 *    const traits = createEntityFeatureFactory(
 *      addLoadEntitiesTrait<Todo>(),
 *      addResetEntitiesStateTrait()
 *    )({
 *      actionsGroupKey: '[Todos]',
 *      featureSelector: createFeatureSelector<TestState>>(
 *        'todos',
 *      ),
 *    });
 * // will generate  the following actions, plus the ones generated by other traits
 * traits.actions.resetTodosState()
 */
export function addResetEntitiesStateTrait(traitConfig = {}) {
    return createTraitFactory({
        key: 'reset',
        config: traitConfig,
        actions: ({ actionsGroupKey, entitiesName, }) => ({
            resetEntitiesState: createAction(`${actionsGroupKey} Reset ${entitiesName} State`),
        }),
        reducer: ({ allActions, initialState }) => createReducer(initialState, on(allActions.resetEntitiesState, () => initialState)),
        effects: ({ allActions }) => {
            var _a;
            class ResetEffect extends TraitEffect {
                //TODO: not sure why Im forced to override this constructor
                // or test wont pass, strangely doesnt happen in other files
                // with similar case like pagination.effects.ts
                constructor(actions$, store) {
                    var _a;
                    super(actions$, store);
                    this.externalReset$ = ((_a = traitConfig === null || traitConfig === void 0 ? void 0 : traitConfig.resetOn) === null || _a === void 0 ? void 0 : _a.length) &&
                        createEffect(() => {
                            return this.actions$.pipe(ofType(...traitConfig === null || traitConfig === void 0 ? void 0 : traitConfig.resetOn), mapTo(allActions.resetEntitiesState()));
                        });
                }
            }
            ResetEffect.ɵfac = i0.ɵɵngDeclareFactory({ minVersion: "12.0.0", version: "12.1.2", ngImport: i0, type: ResetEffect, deps: [{ token: i1.Actions }, { token: i2.Store }], target: i0.ɵɵFactoryTarget.Injectable });
            ResetEffect.ɵprov = i0.ɵɵngDeclareInjectable({ minVersion: "12.0.0", version: "12.1.2", ngImport: i0, type: ResetEffect });
            i0.ɵɵngDeclareClassMetadata({ minVersion: "12.0.0", version: "12.1.2", ngImport: i0, type: ResetEffect, decorators: [{
                        type: Injectable
                    }], ctorParameters: function () { return [{ type: i1.Actions }, { type: i2.Store }]; } });
            return ((_a = traitConfig === null || traitConfig === void 0 ? void 0 : traitConfig.resetOn) === null || _a === void 0 ? void 0 : _a.length) ? [ResetEffect] : [];
        },
    });
}
//# sourceMappingURL=reset.trait.js.map