import { createSelectEntitiesTraitActions } from './select-entities.trait.actions';
import { createSelectEntitiesTraitSelectors } from './select-entities.trait.selectors';
import { loadEntitiesTraitKey, } from '../load-entities/load-entities.model';
import { createTraitFactory } from 'ngrx-traits';
import { createSelectEntitiesInitialState, createSelectEntitiesTraitReducer, } from './select-entities.trait.reducer';
import { createSelectEntitiesTraitMutators } from './select-entities.trait.mutators';
/**
 * Generates ngrx code to add multi selection to a list
 *
 * @example
 * // The following trait config
 *
 * export interface TestState
 * extends EntityAndStatusState<Todo>,MultipleSelectionState{}
 *
 *    const traits = createEntityFeatureFactory(
 *      addLoadEntitiesTrait<Todo>(),
 *      addSelectEntitiesTrait<Todo>()
 *    )({
 *      actionsGroupKey: '[Todos]',
 *      featureSelector: createFeatureSelector<TestState>>(
 *        'todos',
 *      ),
 *    });
 * // will generate  the following actions and selectors, plus the ones generated by other traits
 * traits.actions.selectTodos({id})
 * traits.actions.deselectTodos({id})
 * traits.actions.toggleSectTodos({id})
 * traits.actions.toggleSelectAllTodos()
 * traits.actions.clearTodosSelection()
 *
 * traits.selectors.isAllTodosSelected()
 * traits.selectors.selectTodosIdsSelectedMap()
 * traits.selectors.selectTodosIdsSelectedList()
 * traits.selectors.selectTodosSelectedMap()
 * traits.selectors.selectTodosSelectedList()
 * traits.selectors.selectTotalSelectedTodos()
 */
export function addSelectEntitiesTrait() {
    return createTraitFactory({
        key: 'SelectEntities',
        depends: [loadEntitiesTraitKey],
        actions: ({ actionsGroupKey, entitiesName }) => createSelectEntitiesTraitActions(actionsGroupKey, entitiesName),
        selectors: ({ previousSelectors }) => createSelectEntitiesTraitSelectors(previousSelectors),
        initialState: ({ previousInitialState }) => createSelectEntitiesInitialState(previousInitialState),
        mutators: ({ allSelectors }) => createSelectEntitiesTraitMutators(allSelectors),
        reducer: ({ initialState, allActions, allMutators, allConfigs }) => createSelectEntitiesTraitReducer(initialState, allActions, allMutators, allConfigs),
    });
}
//# sourceMappingURL=select-entities.trait.js.map