import { createTraitFactory, } from 'ngrx-traits';
import { createSelectEntityTraitActions } from './select-entity.trait.actions';
import { createSelectEntityTraitSelectors } from './select-entity.trait.selectors';
import { createSelectEntityInitialState, createSelectEntityTraitReducer, } from './select-entity.trait.reducer';
import { createSelectEntityTraitMutators } from './select-entity.trait.mutators';
import { loadEntitiesTraitKey, } from '../load-entities/load-entities.model';
/**
 * Generates ngrx code to add single selection to a list
 * @param config
 * @param config.selectedId - Default selected id
 *
 * @example
 * // The following trait config
 *
 * export interface TestState
 * extends EntityAndStatusState<Todo>,SelectEntityState{}
 *
 *    const traits = createEntityFeatureFactory(
 *      addLoadEntitiesTrait<Todo>(),
 *      addSelectEntityTrait<Todo>()
 *    )({
 *      actionsGroupKey: '[Todos]',
 *      featureSelector: createFeatureSelector<TestState>>(
 *        'todos',
 *      ),
 *    });
 * // will generate  the following actions and selectors, plus the ones generated by other traits
 * traits.actions.selectTodo({id})
 * traits.actions.deselectTodo()
 * traits.actions.toggleSelectTodo({id})
 *
 * traits.selectors.selectTodoIdSelected()
 * traits.selectors.selectTodoSelected()
 */
export function addSelectEntityTrait(config) {
    return createTraitFactory({
        key: 'singleSelection',
        depends: [loadEntitiesTraitKey],
        config,
        actions: ({ actionsGroupKey, entityName }) => createSelectEntityTraitActions(actionsGroupKey, entityName),
        selectors: () => createSelectEntityTraitSelectors(),
        mutators: () => createSelectEntityTraitMutators(),
        initialState: ({ previousInitialState, allConfigs, }) => createSelectEntityInitialState(previousInitialState, allConfigs),
        reducer: ({ initialState, allActions, allMutators, allConfigs }) => createSelectEntityTraitReducer(initialState, allActions, allMutators, allConfigs),
    });
}
//# sourceMappingURL=select-entity.trait.js.map