import { CrudEntitiesActions, CrudEntitiesConfig } from './crud-entities.model';
/**
 * Generates ngrx code to add, remove update, upsert entities on a list, it also
 * tracks the changes made, helpful for implementing batch updates. The `storeChanges` (false by default),
 * will store for add and update the changed entity props in the property `entityChanges` of the `Change` object.
 * @param config
 * @param config.storeChanges - Will store the changes made, default fals
 *
 * @example
 * // The following trait config
 *
 * export interface TestState
 * extends EntityAndStatusState<Todo>, CrudState<Todo>{}
 *
 *    const traits = createEntityFeatureFactory(
 *      addLoadEntitiesTrait<Todo>(),
 *      addCrudEntitiesTrait<Todo>()
 *    )({
 *      actionsGroupKey: '[Todos]',
 *      featureSelector: createFeatureSelector<TestState>>(
 *        'todos',
 *      ),
 *    });
 * // will generate  the following actions and selectors, plus the ones generated by other traits
 * traits.actions.addTodos(entity1,entity2...)
 * traits.actions.updateTodos({id: id1, changes:{prop1}},{id: id2, changes:{prop2}} ...)
 * traits.actions.upsertTodos(entity1,entity2...)
 * traits.actions.removeAllTodos()
 * traits.actions.clearTodosChanges()
 *
 * traits.selectors.selectTodosChangesList()
 * traits.selectors.selectFilteredTodosChangesList()
 * traits.selectors.selectAllFilteredChanges()
 */
export declare function addCrudEntitiesTrait<Entity>({ storeChanges, }?: CrudEntitiesConfig): import("ngrx-traits").TraitFactory<import("../load-entities/load-entities.model").LoadEntitiesState<Entity> & import("./crud-entities.model").CrudEntitiesState<Entity>, CrudEntitiesActions<Entity>, import("./crud-entities.model").CrudEntitiesSelectors<Entity>, import("./crud-entities.model").CrudEntitiesMutators<Entity>, "crud", CrudEntitiesConfig, import("ngrx-traits").AllTraitConfigs>;
