import { LoadEntitiesState, LoadEntitiesConfig } from './load-entities.model';
/**
 * Generates the ngrx code needed to load a list of entities from the backend
 * with a load[EntitiesName], load[EntitiesName]Success, load[EntitiesName]Fail actions, and selectors to query the
 * entities like selectEntity[EntitiesName]List, selectEntity[EntitiesName]Ids, selectEntity[EntitiesName]Map, and its progress loading
 * with isLoading[EntitiesName],isLoading[EntitiesName]Success, isLoading[EntitiesName]Fail. This trait is the base for all other traits related
 * to a list of entities, the other will call loadEntities when needing data.
 * @param traitConfig - Config object fot the trait factory
 * @param traitConfig.selectId - Function that returns the id of an entity
 * @param traitConfig.sortComparer - Default sort function for to @ngrx/entity EntityAdapter
 *
 * @example
 * // The following trait config
 *
 * export interface TestState
 * extends LoadEntitiesState<Todo>{}
 *
 *    const traits = createEntityFeatureFactory(
 *      {entityName: 'Todo'},
 *      addLoadEntitiesTrait<Todo>(),
 *    )({
 *      actionsGroupKey: '[Todos]',
 *      featureSelector: createFeatureSelector<TestState>>(
 *        'todos',
 *      ),
 *    });
 * // will generate the actions and selectors
 * traits.actions.loadTodos()
 * traits.actions.loadTodosSuccess({entities: todos})
 * traits.actions.loadTodosFail();
 * traits.selectors.selectTodosList
 * traits.selectors.selectTodosMap
 * traits.selectors.selectTodosIds
 * traits.selectors.selectTodosTotal
 * traits.selectors.isLoadingTodos
 * traits.selectors.isSuccessTodosSuccess
 * traits.selectors.isFailTodosFail
 */
export declare function addLoadEntitiesTrait<Entity>(traitConfig?: Omit<LoadEntitiesConfig<Entity>, 'adapter'>): import("ngrx-traits").TraitFactory<LoadEntitiesState<Entity>, import("./load-entities.model").LoadEntitiesActions<Entity>, import("./load-entities.model").LoadEntitiesSelectors<Entity>, import("./load-entities.model").LoadEntitiesMutators<Entity>, "loadEntities", LoadEntitiesConfig<Entity>, import("ngrx-traits").AllTraitConfigs>;
