import { GenericActionCreator } from '../load-entities';
/**
 * Generates the ngrx code needed to reset the current state to the initial state.
 * @param traitConfig - Config object fot the trait factory
 * @param traitConfig.resetOn - set an extra action which will also trigger a reset state,
 * useful if you want to create an action that reset several features states
 *
 * @example
 * // The following trait config
 *
 * export interface TestState
 * extends EntityAndStatusState<Todo>,FilterState<TodoFilter>{}
 *
 *    const traits = createEntityFeatureFactory(
 *      addLoadEntitiesTrait<Todo>(),
 *      addResetEntitiesStateTrait()
 *    )({
 *      actionsGroupKey: '[Todos]',
 *      featureSelector: createFeatureSelector<TestState>>(
 *        'todos',
 *      ),
 *    });
 * // will generate  the following actions, plus the ones generated by other traits
 * traits.actions.resetTodosState()
 */
export declare function addResetEntitiesStateTrait(traitConfig?: {
    resetOn?: GenericActionCreator[];
}): import("ngrx-traits").TraitFactory<{}, {
    resetEntitiesState: import("@ngrx/store").ActionCreator<string, () => import("@ngrx/store/src/models").TypedAction<string>>;
}, {}, {}, "reset", {
    resetOn?: GenericActionCreator[] | undefined;
}, import("ngrx-traits").AllTraitConfigs>;
