import { SelectEntitiesSelectors, SelectEntitiesActions } from './select-entities.model';
/**
 * Generates ngrx code to add multi selection to a list
 *
 * @example
 * // The following trait config
 *
 * export interface TestState
 * extends EntityAndStatusState<Todo>,MultipleSelectionState{}
 *
 *    const traits = createEntityFeatureFactory(
 *      addLoadEntitiesTrait<Todo>(),
 *      addSelectEntitiesTrait<Todo>()
 *    )({
 *      actionsGroupKey: '[Todos]',
 *      featureSelector: createFeatureSelector<TestState>>(
 *        'todos',
 *      ),
 *    });
 * // will generate  the following actions and selectors, plus the ones generated by other traits
 * traits.actions.selectTodos({id})
 * traits.actions.deselectTodos({id})
 * traits.actions.toggleSectTodos({id})
 * traits.actions.toggleSelectAllTodos()
 * traits.actions.clearTodosSelection()
 *
 * traits.selectors.isAllTodosSelected()
 * traits.selectors.selectTodosIdsSelectedMap()
 * traits.selectors.selectTodosIdsSelectedList()
 * traits.selectors.selectTodosSelectedMap()
 * traits.selectors.selectTodosSelectedList()
 * traits.selectors.selectTotalSelectedTodos()
 */
export declare function addSelectEntitiesTrait<Entity>(): import("ngrx-traits").TraitFactory<import("../load-entities/load-entities.model").LoadEntitiesState<Entity> & import("./select-entities.model").SelectEntitiesState, SelectEntitiesActions, SelectEntitiesSelectors<Entity>, import("./select-entities.model").SelectEntitiesMutators<Entity>, "SelectEntities", unknown, import("ngrx-traits").AllTraitConfigs>;
