import { SelectEntityActions, SelectEntityConfig } from './select-entity.model';
/**
 * Generates ngrx code to add single selection to a list
 * @param config
 * @param config.selectedId - Default selected id
 *
 * @example
 * // The following trait config
 *
 * export interface TestState
 * extends EntityAndStatusState<Todo>,SelectEntityState{}
 *
 *    const traits = createEntityFeatureFactory(
 *      addLoadEntitiesTrait<Todo>(),
 *      addSelectEntityTrait<Todo>()
 *    )({
 *      actionsGroupKey: '[Todos]',
 *      featureSelector: createFeatureSelector<TestState>>(
 *        'todos',
 *      ),
 *    });
 * // will generate  the following actions and selectors, plus the ones generated by other traits
 * traits.actions.selectTodo({id})
 * traits.actions.deselectTodo()
 * traits.actions.toggleSelectTodo({id})
 *
 * traits.selectors.selectTodoIdSelected()
 * traits.selectors.selectTodoSelected()
 */
export declare function addSelectEntityTrait<Entity>(config?: SelectEntityConfig): import("ngrx-traits").TraitFactory<import("../load-entities/load-entities.model").LoadEntitiesState<Entity> & import("./select-entity.model").SelectEntityState, SelectEntityActions, import("./select-entity.model").SelectEntitySelectors<Entity>, import("./select-entity.model").SelectEntityMutators<unknown>, "singleSelection", SelectEntityConfig, import("ngrx-traits").AllTraitConfigs>;
