import { SortEntitiesActions, SortEntitiesConfig, SortEntitiesSelectors } from './sort-entities.model';
/**
 * Generates ngrx code to sort locally or remotely a list of entities
 * @param config
 * @param config.defaultSort - Required, Default entity prop for the sort
 * @param config.remote - Optional field, default to false, when true disables local
 * sorting and every sort action call will now trigger a loadEntities action and the backend
 * should sort the data, use selectSort in the effect that call backend to get the requested sort,
 * when false all sorting is done in memory when the sort action is fired
 *
 * @example
 * // The following trait config
 * export interface TestState
 * extends EntityAndStatusState<Todo>, SortState<Todo>{}
 *
 *    const traits = createEntityFeatureFactory(
 *      addLoadEntitiesTrait<Todo>(),
 *      addSortEntitiesTrait<Todo>({
 *        defaultSort: {active:'id', direction:'asc'}
 *      })
 *    )({
 *      actionsGroupKey: '[Todos]',
 *      featureSelector: createFeatureSelector<TestState>>(
 *        'todos',
 *      ),
 *    });
 * // will generate  the following actions and selectors, plus the ones generated by other traits
 * traits.actions.sortTodos({active:'id', direction:'desc'})
 * traits.actions.resetTodosSort()
 *
 * traits.selectors.selectTodosSort()
 */
export declare function addSortEntitiesTrait<Entity>({ remote, defaultSort, }: SortEntitiesConfig<Entity>): import("ngrx-traits").TraitFactory<import("../load-entities/load-entities.model").LoadEntitiesState<Entity> & import("./sort-entities.model").SortEntitiesState<Entity>, SortEntitiesActions<Entity>, SortEntitiesSelectors<Entity>, import("./sort-entities.model").SortEntitiesMutators<Entity>, "sort", SortEntitiesConfig<Entity>, import("ngrx-traits").AllTraitConfigs>;
